# bpcca.py

# Barnett-Preisendorfer Canonical Correlation Analysis
# This program performs a previous EOF decomposition in order to
# retain only the most important modes and then applies CCA.
# The output are NumPy arrays containing the canonical patterns,
# the expansion coefficients and the correlations.
# For further details and notation:
# Bretherton et al. "An Intercomparison of Methods for Finding Coupled 
# Patterns in Climate Data", Journal of Climate, vol. 5 (1992), p541

# Copyright (C) 2000, Jon Saenz, Jesus fernandez and Juan Zubillaga

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# jff20001107

from pyclimate.mvarstatools import *
from pyclimate.svdeofs import *
from LinearAlgebra import *
from Numeric import *

mm = matrixmultiply

def getneofs(lbd, bppercent=70):
  percentlbd = 100*lbd/add.reduce(lbd)
  underlimit = less_equal(add.accumulate(percentlbd), float(bppercent))
  return 1 + add.reduce(underlimit)

class bpcca:
  def __init__(self, s, z, retainedeofs=None):
    self.sfield2d = len(s.shape)==2
    if not self.sfield2d:
      s, self.oldsshape = unshape(s)
    self.zfield2d = len(z.shape)==2
    if not self.sfield2d:
      z, self.oldzshape = unshape(z)
    # First the EOF calculus
    # If the field dimensions are:
    # s -> t x ns
    # z -> t x nz
    # we obtain: (where p = min(t,ns)
    # pcs  -> t  x p 
    # lbds -> p  x p
    # eofs -> ns x p
    pcs, lbds, eofs = svdeofs(array(s,Float64), 1)
    self.totsvar = add.reduce(lbds)
    # p = min(t,nz)
    # pcz  -> t  x p 
    # lbdz -> p  x p
    # eofz -> nz x p
    pcz, lbdz, eofz = svdeofs(array(z,Float64), 1)
    self.totzvar = add.reduce(lbdz)
    # Restriction to an EOFs subspace...
    if not retainedeofs:
      n1 = getneofs(lbds) 
      n2 = getneofs(lbdz)
    else:
      n1 = retainedeofs[0]
      n2 = retainedeofs[1]
    self.n1 = n1
    self.n2 = n2
    pcs = pcs[:,:n1]    # t x n1
    pcz = pcz[:,:n2]    # t x n2
    lbds = lbds[:n1]    # n1
    lbdz = lbdz[:n2]    # n2
    eofs = eofs[:,:n1]  # ns x n1
    eofz = eofz[:,:n2]  # nz x n2
    covpcsz = mm(transpose(pcs), pcz)/float(len(pcs))
    left, sigma, t_right = singular_value_decomposition(covpcsz)
    right = transpose(t_right)
    s_recons = mm(pcs, transpose(eofs))
    z_recons = mm(pcz, transpose(eofz))
    covss_recons = mm(transpose(s_recons), s_recons) / len(s_recons)
    covzz_recons = mm(transpose(z_recons), z_recons) / len(z_recons)
    # Obtaining the patterns and coefficients...
    # min(n1,n2) is shown as n0
    # A partial reconstruction of the original fields is achieved by:
    # s_n0 = mm(a, transpose(p))
    # z_n0 = mm(b, transpose(q))
    self.p_adjoint = mm(eofs/lbds[NewAxis,:], left)     # ns x n0
    self.q_adjoint = mm(eofz/lbdz[NewAxis,:], right)    # nz x n0
    self.p = mm(covss_recons, self.p_adjoint)           # ns x n0
    self.q = mm(covzz_recons, self.q_adjoint)           # nz x n0
    self.a = mm(pcs, left)                         # t  x n0
    self.b = mm(pcz, right)                        # t  x n0
    self.corr = sigma                              # n0

  def getvariancefractions(self):
    ndimsp = len(self.p.shape)
    ndimsq = len(self.q.shape) 
    vars = self.p * self.p
    varz = self.q * self.q
    for i in range(ndimsp-1):
      vars = add.reduce(vars)
    for i in range(ndimsq-1):
      varz = add.reduce(varz)
    return vars / self.totsvar, varz / self.totzvar

  def leftpatterns(self):
    if self.sfield2d:
      return self.p
    else:
      return deunshape(self.p, self.oldsshape[1:]+self.p.shape[-1:])

  def rightpatterns(self):
    if self.zfield2d:
      return self.q
    else:
      return deunshape(self.q, self.oldzshape[1:]+self.q.shape[-1:])

  def leftexpcoeffs(self):
    return self.a

  def rightexpcoeffs(self):
    return self.b

  def correlation(self):
    return self.corr


