# svdeofs.py
#
# Given a dataset as read from readdat, that is,
# a matrix NxM, with N the number of samples and M the number of channels
# or spatial samples, these functions compute the unrotated EOF
# decomposition of the field, the principal components
# and some utility routines
#
# 
# Copyright (C) 2000, Jon Saenz, Jesus fernandez and Juan Zubillaga
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 

from Numeric import *

import LinearAlgebra
import sys
import whrandom
import Scientific.Statistics
import pyclimate.mvarstatools
import pyclimate.pydcdflib
import pyclimate.pyclimateexcpt
import pyclimate.mctest
from pyclimate.tools import *

mctest=pyclimate.mctest
mt=pyclimate.mvarstatools
pex=pyclimate.pyclimateexcpt
mm=matrixmultiply

# Derive the EOFs from a dataset by means of the SVD decomposition of the
# data matrix (See Jackson, 1991 and Preisendorfer, 1987), avoiding the
# problems associated to a singular correlation matrix
# dataset is an nxm array of data.
# n (rows) -> Samples
# m (columns) -> Data channels
# If p=min(n,m) the output dimensions are: pcs(nxp)
#                                          L  (pxp)
#                                          E  (mxp)
def svdeofs(dataset, pcscaling=0):
	"""\
  Calculates de EOF decomposition of a field.
  
  svdeofs(dataset, pcscaling=0)
  dataset -- NumPy array containing the field to be decomposed. First dimension must be time.
  pcscaling -- sets the scale factor of the PCs: 0 - Eigenvalues are PC variances
                                                     Orthonormal EOFs
                                                 1 - PC variance = 1
                                                     Orthogonal EOFs

  If the field has more than one spatial dimension it can be processed anyway
  and each EOF can be recovered as "generalized columns": EOFs[..., eofnumber]
  Returns a tuple containing: PCs, eigenvalues, EOFs
  """
	residual=mt.center(dataset)
	field2d = len(dataset.shape)==2
	if not field2d:
		residual, oldshape = unshape(residual)
	A,Lh,E=LinearAlgebra.singular_value_decomposition(residual)
	# The eigenvalues from SVD routines are powered to 1/2, thus: square
	# the vector.
	# Moreover, in order to be able to compare these eigenvalues to the 
	# ones obtained from the direct eigenvalue problem with a non-singular
	# covariance matrix (i.e. dsyev() LAPACK routine),
	# the eigenvalues have to be divided by a 
	# constant factor, the number of the elements in the time series.
	# This way, the eigenvalues are variances!!
	# (Defined not taking into account that the centered data has 
	# one degree of freedom less)
	# First BUG Fixed (JFF 20001115):
	# A "-1" appeared here before version 1.1
	# (it has been removed for consistency)
 	normfactor=float(len(residual))
	L=Lh*Lh/normfactor
	# E is returned transposed
	E=transpose(E)
	pcs = A*Lh
	if pcscaling == 0:
		# E orthonormal
		# pc variances are the eigenvalues L
		pass
	elif pcscaling == 1:
		# E orthogonal but not orthonormal
		# unity pc variances
		E = E * sqrt(L)[NewAxis,:]
		pcs = pcs / sqrt(L)[NewAxis,:]
	else:
		raise pex.ScalingError(pcscaling)
	if not field2d:
		E = deunshape(E, oldshape[1:]+E.shape[-1:])
	return pcs,L,E

def getgencol(a, ncol=0):
	return a[...,ncol]

def pcseriescorrelation(pcs, eofs, dataset):
	"""\
  Calculates the correlation between the PCs and time series at each grid point.

  def pcseriescorrelation(pcs, eofs, dataset):
  pcs -- the PCs as returned by <svdeofs()>
  eofs -- the EOFs as returned by <svdeofs()>
  dataset -- the dataset

  Returns an array which "generalized columns" are the correlation fields of the
  original time series with each PC.
        """
	residual = mt.center(dataset)
	datastd = sqrt(add.reduce(residual*residual)/float(len(residual)))
	pcsstd = sqrt(add.reduce(pcs*pcs)/float(len(pcs)))
	return eofs * pcsstd / datastd[...,NewAxis] 

def getvariancefraction(lambdas):
	return lambdas/add.reduce(lambdas)

def bartletttest(lambdas,samples):
	p=len(lambdas)
	theshape=(p-1,)
	chis=zeros(theshape,Float64)
	pchis=zeros(theshape,Float64)
	nu=samples-1
	# This test will fail when lambda[i]<=0!!!
	mask=less_equal(lambdas,0.0)
	maskedlambdas=lambdas*logical_not(mask)+mask
	loglambdas=log(maskedlambdas)
	for k in xrange(p-1):
		cdf=.5*(p-k-1)*(p-k+2)
		if mask[k]==1:
			# "Transformed" eigenvalues, set a probability of 1
			pchis=pchis*logical_not(mask)+mask
			chis[k]=0.0
		else:
			chis[k]=nu*((p-k)* \
				log(add.reduce(maskedlambdas[k:])/(p-k)) \
				-add.reduce(loglambdas[k:]))
			pchis[k]=getchiprob(chis[k],cdf)
	return (chis,pchis)

def getchiprob(chival,dof):
	chi=pyclimate.pydcdflib.CDFChi()
	chi.which=1
	chi.status=0
	chi.x=float(chival)
	chi.df=dof
	pyclimate.pydcdflib.pycdfchi(chi)
	if chi.status==0:
		prob=chi.p
	else:
		prob=0
	return prob


def northtest(lambdas,tsamples):
	factor=sqrt(2./tsamples)
	errors=array(lambdas)*factor
	return errors


# Test the leading master EOFs obtained from the complete sample and
# input in eofs by means of a Monte Carlo test based on making
# subsamples with length members in each 
def mctesteofs(dataset,eofs,subsamples,length):
	eofnumber=eofs.shape[-1]
	theccoefs=zeros((subsamples,)+(eofnumber,),Float64)
	for isample in xrange(subsamples):
		subslist=mctest.getrandomsubsample(length,len(dataset))
		subsample=take(dataset,subslist,0)
		z,lambdas,eofdot=svdeofs(subsample)
		for ieof in xrange(eofnumber):
			theccoefs[isample,ieof]= \
				pyclimate.mvarstatools.congruence(
					eofdot[...,ieof],eofs[...,ieof]
				)
	return theccoefs

# jff20001108
